/***********************************************************************
*                    SEGGER Microcontroller GmbH                       *
*                        The Embedded Experts                          *
************************************************************************
*                                                                      *
*                  (c) SEGGER Microcontroller GmbH                     *
*                        All rights reserved                           *
*                          www.segger.com                              *
*                                                                      *
************************************************************************
*                                                                      *
************************************************************************
*                                                                      *
*                                                                      *
*  Licensing terms                                                     *
*                                                                      *
* Redistribution and use in source and binary forms, with or without   *
* modification, are permitted provided that the following conditions   *
* are met:                                                             *
*                                                                      *
* 1. Redistributions of source code must retain the above copyright    *
* notice, this list of conditions and the following disclaimer.        *
*                                                                      *
* 2. Redistributions in binary form must reproduce the above           *
* copyright notice, this list of conditions and the following          *
* disclaimer in the documentation and/or other materials provided      *
* with the distribution.                                               *
*                                                                      *
*                                                                      *
* THIS SOFTWARE IS PROVIDED BY COPYRIGHT HOLDER "AS IS" AND ANY        *
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    *
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR   *
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDER BE        *
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,     *
* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,             *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   *
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  *
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         *
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH.    *
* DAMAGE.                                                              *
*                                                                      *
************************************************************************

-------------------------- END-OF-HEADER -----------------------------

File    : RAMCODE_SPIFI_Lib.h
Purpose :
*/

#ifndef SEGGER_SPIFI_LIB_H
#define SEGGER_SPIFI_LIB_H

#include "RAMCODE_Intern.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

//
// Flags passed by generic layer to SPIFI lib on Init()
//
#define SEGGER_SPIFI_LIB_SESSION_FLAG_IS_DUAL_SPIFI         (1uL << 0)          // We are talking to two SPIFI flashes in parallel (called dual SPIFI)
//
// Flags for callbacks called by SPIFI module
//
//  [3:0]: Should not be used. These are from the old QSPI lib
//  [4:4]: program using quad mode
//  [5:5]: dual SPI
//  [6:6]: Lib requests to put SPIFI controller into 4-byte address mode while operating in memory mapped mode
//  [8:7]: Access type
//           0 : read
//           1 : write
//           2 : verify
//           3 : reserved
#define SEGGER_SPIFI_LIB_CB_FLAG_PROG_IN_QUAD_MODE          (1uL << 4)
#define SEGGER_SPIFI_LIB_CB_FLAG_IS_DUAL_SPIFI              (1uL << 5)
#define SEGGER_SPIFI_LIB_CB_FLAG_ENABLE_MEMMAPPED_4BMODE    (1uL << 6)
#define SEGGER_SPIFI_LIB_CB_FLAG_DATA_READ                  (0uL << 7)
#define SEGGER_SPIFI_LIB_CB_FLAG_DATA_WRITE                 (1uL << 7)
#define SEGGER_SPIFI_LIB_CB_FLAG_DATA_VERIFY                (2uL << 7)

#define SEGGER_SPIFI_LIB_CB_FLAG_DATA_MASK                  (3uL << 7)

#define SEGGER_SPIFI_LIB_ERR_UNSPEC                         (-1)
#define SEGGER_SPIFI_LIB_ERR_TOO_MANY_CONT_BYTES            (-2)
#define SEGGER_SPIFI_LIB_ERR_UNKNOWN_FLASH_ID               (-3)
#define SEGGER_SPIFI_LIB_ERR_COMM_ERR                       (-4)
#define SEGGER_SPIFI_LIB_ERR_NOT_SUFF_CFI_BYTES             (-5)
#define SEGGER_SPIFI_LIB_ERR_MAX_OFF_EXCEEDED               (-6)

#define SEGGER_SPIFI_LIB_IPARAM_MAX_NUM_DATA_BYTES_SHIFT_QSPIC (0)  // QSPIC = QSPI controller. Max. num data bytes shift for QSPI controller. Requested by lib on program

/*********************************************************************
*
*       Types
*
**********************************************************************
*/

typedef struct {
  //
  // Pub struct is filled by caller and not modified by callee
  // NOTE!!!!!!!!!!!: When changing anything in the struct, make sure that RAMCode_SPIFI_LibContext.asm is also changed accordingly (best way: compile a const instance of the struct and take the struct size from the list file)
  //
  struct {
    //
    // Mandatory callbacks
    //
    // For dual SPI and _PROG_IN_QUAD_MODE == 0: Bytes are sent alternating to flashes: pData == F0B0 F1B0 F0B1 F1B1 ...
    //                  _PROG_IN_QUAD_MODE == 1: Each flash gets 4-bits of a byte
    // For dual SPI: NumBytesData has to be a multiple of 2 bytes. We will get data alternating from each flash: pData == F0B0 F1B0 F0B1 F1B1 ...
    //
    int  (*pfExecCmd)(U32 Cmd, U32 Addr, U32 NumBytesAddr, void* pData, U32 NumBytesData, U32 cbFlags, U32* pRAMCodeTmpData);  // If set, pfReadId() must also be set
    //
    // Optional callbacks
    //
    // pfExecCmd:
    //   cbFlags: _FLAG_CS_START_STATE and _FLAG_CS_END_STATE do not need to be checked. It is guaranteed that start state of nCS is 0 and end state is 1 (indicating a complete SPI transfer)
    //   pData:   May be a buffer to hold read data or a buffer that holds data to be written. Write is indicated by the existence of SEGGER_SPIFI_LIB_CB_FLAG_IS_DATA_WRITE in cbFlags
    //
    // pfGetParam:
    //   Optional func to get target specific parameters like SPI buffer limits, so programming a page etc. may be split by the lib
    //   Return < 0: No specifics available for specified parameter
    //
    void (*pfOptEnterLeaveSPIMode)(U32* pRAMCodeTmpData, int IsLeave, U32 cbFlags, U8 dummyClks);
    int  (*pfGetParam)(U32 iParam, U32* pVal);
    //
    // Other params
    //
    U32 RAMCodeTmpData;                                                         // Memory for temporary data storage, initially provided by Erase(), Program() etc. calls and passed through SPIFI lib back to specific part if some things need to be remembered accross function calls
  } Pub;
  //
  // Int struct is filled by callee and must not be modified by caller
  //
  struct {
    U8 acData[64];
    U32 NumSectors;
    U32 SessionFlags;                                           // Or-combination of SEGGER_SPIFI_LIB_SESSION_FLAG_
    U8 DummyCycles;
    unsigned int MinAlignProg       : 5;
    unsigned int SectorSizeShift    : 5;
    unsigned int StatRegFormat      : 4;
    unsigned int Type4BMode         : 3;
    unsigned int QEModeEnabled      : 1;
    unsigned int Only4ByteAddress   : 1;
    unsigned int FastRDMode         : 3;
  } Int;
}   SEGGER_SPIFI_LIB_CONTEXT;

/*********************************************************************
*
*       Function declarations
*
**********************************************************************
*/

int  SEGGER_SPIFI_LIB_Init     (SEGGER_SPIFI_LIB_CONTEXT* pContext, U8 SessionFlags);
int  SEGGER_SPIFI_LIB_UnInit   (SEGGER_SPIFI_LIB_CONTEXT* pContext, U8 SessionFlags);
int  SEGGER_SPIFI_LIB_Program  (SEGGER_SPIFI_LIB_CONTEXT* pContext, U32 Addr, U32 NumBytes, const void* pData);
int  SEGGER_SPIFI_LIB_Erase    (SEGGER_SPIFI_LIB_CONTEXT* pContext, U32 SectorAddr, U32 NumSectors);

#endif // SEGGER_SPIFI_LIB_H

/****** End Of File *************************************************/
